use axum::{
    extract::{Path, Query},
    http::header,
    response::IntoResponse,
    routing::get,
    Router,
};
use std::collections::HashMap;
use tera::Tera;

#[tokio::main]
async fn main() {
    // build our application with a single route
    let app = Router::new()
        .route("/", get(index))
        .route("/search", get(search))
        .route("/paper/:id", get(paper));

    // run it with hyper on localhost:3000
    axum::Server::bind(&"0.0.0.0:3000".parse().unwrap())
        .serve(app.into_make_service())
        .await
        .unwrap();
}

async fn index() -> impl IntoResponse {
    // Create a new Tera instance and add a template from a string
    let tera = tera_create();
    // Prepare the context with some data
    let context = tera::Context::new();

    // Render the template with the given context
    let rendered = tera.render("index.html", &context).unwrap();

    header(rendered)
}

async fn search(Query(params): Query<HashMap<String, String>>) -> impl IntoResponse {
    let tera = tera_create();
    // Prepare the context with some data
    let mut context = tera::Context::new();
    context.insert("query", params.get("q").unwrap());

    let rendered = tera.render("search.html", &context).unwrap();

    header(rendered)
}

async fn paper(Path(paper_id): Path<u64>) -> impl IntoResponse {
    let response = reqwest::get(format!(
        "https://www.muenchen-transparent.de/oparl/v1.0/paper/{0}",
        paper_id
    ))
    .await
    .unwrap()
    .json::<HashMap<String, String>>()
    .await
    .unwrap();

    let tera = tera_create();
    let mut context = tera::Context::new();
    context.insert("paper_id", &paper_id);
    context.insert("paper_name", &response.get("name").unwrap());

    let rendered = tera.render("paper.html", &context).unwrap();

    header(rendered)
}

fn tera_create() -> Tera {
    let tera = match Tera::new("templates/**/*.html") {
        Ok(tera) => tera,
        Err(error) => {
            println!("Parsing error(s): {}", error);
            ::std::process::exit(1);
        }
    };

    tera
}

fn header(rendered: impl ToString) -> impl IntoResponse {
    (
        [(header::CONTENT_TYPE, "text/html; charset=utf-8")],
        rendered.to_string(),
    )
}
